## Créé le 18/6/2020 ##

######################################################
#  ___         __    __  ___         __   ___  ____  #
#  |__| |  |  /  \  /    |__| |  |  /  \  |__| |     #
#  |    |--| |    | ---- |    |--| |    | |\   |--   #
#  |    |  |  \__/  ___/ |    |  |  \__/  | \  |___  #
#                                                    #
######################################################

##
#
##
proc Katyusha_Configurations_liste_elements_config {} {
    return [list "LANG" "RESOLUTION" "STATS" "NOM_BDD_DEFAUT" "SGBD_DEFAUT" "DROP_DEFAUT" "TAILLE_CANVAS"]
}

proc Katyusha_Configurations_init {} {
    global rep_configs
    global rpr
    global nom_script
    global rep_mcd
    
    ##
    # Créé les répertoires de configuration s'il n'existent pas
    ##
    if {![file exists "~/.phosphore"]} {
        file mkdir "~/.phosphore"
    }
    if {![file exists $rep_configs]} {
        file mkdir $rep_configs
    }
    # Copie le fichier par défaut si le fichier de configuration n'existe pas
    if {![file exists "$rep_configs/katyusha.conf"]} {
        file copy "$rpr/configs/defaut.conf" "$rep_configs/katyusha.conf"
    } else {
        # Si le fichier existe, vérifie que toutes les données sont présentes
        set fp [open "$rep_configs/katyusha.conf" "r"]
        set contenu [read $fp]
        close $fp
        foreach element [Katyusha_Configurations_liste_elements_config] {
            if {[string first $element $contenu] < 0} {
                file delete "$rep_configs/katyusha.conf"
                file copy "$rpr/configs/defaut.conf" "$rep_configs/katyusha.conf"
            }
        }
    }
    if {![file exists "$rep_configs/recents"]} {
        file copy "$rpr/configs/recents" "$rep_configs/recents"
    }
    
    # Répertoire par défaut des projets Katyusha! MCD
    set rep_mcd "~/Documents/Katyusha_projets"
    # Si il n'existe pas, le créer
    if {![file exists $rep_mcd]} {
        file mkdir $rep_mcd
    }
    # Nom par défaut du script SQL généré
    set id_script [llength [glob -nocomplain -dir $rep_mcd "projet*.sql"]]
    set nom_script "projet$id_script.sql"
}

proc Katyusha_Configurations_charge {rep_main rep_configs} {
    global CONFIGS
    global MCD
    # Charge les configurations
    # Ouvre le fichier en lecture
    set fp [open "$rep_configs/katyusha.conf" "r"]
    set stream [read $fp]
    close $fp
    # Balyage des lignes
    foreach ligne [split $stream "\n"] {
        set clef [lindex [split $ligne ":"] 0]
        set valeur [lindex [split $ligne ":"] 1]
        set CONFIGS($clef) $valeur
    }
    set MCD(nom) $CONFIGS(NOM_BDD_DEFAUT)
    set MCD(sgbd) $CONFIGS(SGBD_DEFAUT)
    set MCD(drop) $CONFIGS(DROP_DEFAUT)
}

proc Katyusha_Configurations_packages {} {
    set noms [dict create "TDBC" "tdbc"]
    foreach {k v} $noms {
        Katyusha_Configurations_package $k $v
    }
}

proc Katyusha_Configurations_package {nom nom_tcl} {
    puts -nonewline "..............$nom"
    if {[catch {package require $nom_tcl}]} {
        puts " # KO"
    } else {
        puts " # OK!"
    }
}

proc Katyusha_Configurations_resolution {} {
    global CONFIGS
    
    # Si la résolution est en mode automatique
    if {$CONFIGS(RESOLUTION) == "auto"} {
        # On récupère la taille de l'écran
        set x [winfo screenwidth .]
        set y [winfo screenheight .]
        set y [expr $y - 80]
    } else {
        set x [lindex [split $CONFIGS(RESOLUTION) "x"] 0]
        set y [lindex [split $CONFIGS(RESOLUTION) "x"] 1]
    }
    return [list $x $y]
}

##
# Retourne une liste des langues disponibles
##
proc Katyusha_Configurations_liste_langues {} {
    global rpr
    
    set liste_langues [list]
    set langues [glob -nocomplain -dir "$rpr/locale" "*.tcl"]
    foreach langue $langues {
        set nom_fichier [lindex [split $langue "/"] 2]
        set code_langue [lindex [split $nom_fichier "."] 0]
        set fp [open $langue "r"]
        set contenu_fichier [read $fp]
        close $fp
        set premiere_ligne [lindex [split $contenu_fichier "\n"] 0]
        set nom_langue [lindex [split $premiere_ligne " "] 1]
        lappend liste_langues "$code_langue - $nom_langue"
    }
    return $liste_langues
}

##
# Enregistre les nouveaux paramètres de la base de données, ils doivent avoir été contrôlés avant
##
proc Katyusha_Configurations_MCD {nom liste_sgbd drop_base} {
    global MCD
    # Nom
    if {$nom != $MCD(nom) && $nom != ""} {
        set MCD(nom) $nom
    }
    # Liste des SGBD
    if {[lindex $liste_sgbd 0] != ""} {
        set MCD(sgbd) $liste_sgbd
    }
    # Drop?
    if {$drop_base != $MCD(drop) && $drop_base != ""} {
        set MCD(drop) $drop_base
    }
}

##
# Enregistre les préférences de Katyusha MCD
##
proc Katyusha_Configurations_sauve {langue} {
    global CONFIGS
    global rep_configs
    
    set CONFIGS(LANG) [lindex [split $langue " - "] 0]
    set conf ""
    
    foreach element [Katyusha_Configurations_liste_elements_config] {
        set conf "$conf$element:$CONFIGS($element)\n"
    }
    
    #set conf "LANG:$CONFIGS(LANG)\nRESOLUTION:$CONFIGS(RESOLUTION)\nSTATS:$CONFIGS(STATS)\nNOM_BDD_DEFAUT:$CONFIGS(NOM_BDD_DEFAUT)\nSGBD_DEFAUT:$CONFIGS(SGBD_DEFAUT)\nDROP_DEFAUT:$CONFIGS(DROP_DEFAUT)\nTAILLE_CANVAS:$CONFIGS(TAILLE_CANVAS)"
    
    # Ouvre le fichier en écriture
    set fp [open "$rep_configs/katyusha.conf" "w+"]
    set stream [read $fp]
    puts $fp $conf
    close $fp
}
