## Créé le 5/5/2020 ##

######################################################
#  ___         __    __  ___         __   ___  ____  #
#  |__| |  |  /  \  /    |__| |  |  /  \  |__| |     #
#  |    |--| |    | ---- |    |--| |    | |\   |--   #
#  |    |  |  \__/  ___/ |    |  |  \__/  | \  |___  #
#                                                    #
######################################################


proc Katyusha_GenerationSQL_format_cotes {valeur} {
    if {[string is entier $valeur] == 0 && [string is boolean $valeur] == 0 && [string is double $valeur] == 0 && [string is integer $valeur] == 0} {
        set res "'$valeur'"
    } else {
        set res $valeur
    }
    return $res
}

proc Katyusha_GenerationSQL_format_complement_type_enum {ctype} {
    set res ""
    set liste_enum [split $ctype ";"]
    foreach enum $liste_enum {
        if {$res == ""} {
            set res [Katyusha_GenerationSQL_format_cotes $enum]
        } else {
            set res "$res, [Katyusha_GenerationSQL_format_cotes $enum]"
        }
    }
    return $res
}

proc Katyusha_GenerationSQL_sql_taille_attribut {attribut} {
    set type_attribut [string tolower [dict get $attribut "type"]]
    set taille_attribut [dict get $attribut "taille"]
    set sql ""
    if {$type_attribut == "char" || $type_attribut == "varchar"} {
        if {$taille_attribut == 0} {
            set sql "(255)"
        } else {
            set sql "($taille_attribut)"
        }
    }
    return $sql
}

##
# Construit le script SQL d'un attribut
##
proc Katyusha_GenerationSQL_attribut {id attribut sgbd} {
    set SQL ""
    set nom_attribut [dict get $attribut "nom"]
    set type_attribut [string toupper [dict get $attribut "type"]]
    set ctype_attribut [dict get $attribut "complement_type"]
    set taille_attribut [dict get $attribut "taille"]
    set pk_attribut [dict get $attribut "pk"]
    set null_attribut [dict get $attribut "null"]
    set auto_attribut [dict get $attribut "auto"]
    set valeur_attribut [dict get $attribut "valeur"]
    
    set sql_nom $nom_attribut
    
    # Si l'attribut est en incrémentation automatique, le type est modifié selon le SGBD
    if {$auto_attribut == 1} {
        set type_attribut [Katyusha_SQL_auto_increment_type $sgbd]
    }
    set sql_type " [string toupper $type_attribut]"
    
    if {$auto_attribut == 1} {
        set sql_type "$sql_type [lindex [Katyusha_SQL_auto_increment $sgbd] 0]"
        set null_attribut 0
    }
    
    if {$ctype_attribut != ""} {
        set sql_ctype "($ctype_attribut)"
    } else {
        set sql_ctype ""
    }
    set sql_taille [Katyusha_GenerationSQL_sql_taille_attribut $attribut]
    
    # Détermine si un attribut est nul
    if {$valeur_attribut != "null" || $pk_attribut == 1 || $null_attribut == 0} {
        set sql_null " NOT NULL"
    } else {
        set sql_null ""
    }
    
    # Valeur par défaut de l'attribut
    if {$auto_attribut == 0 && $valeur_attribut != "" && $valeur_attribut != "null"} {
        set sql_valeur " DEFAULT [Katyusha_GenerationSQL_format_cotes $valeur_attribut]"
    } else {
        set sql_valeur ""
    }
    set SQL "$sql_nom$sql_type$sql_taille$sql_ctype$sql_null$sql_valeur"
    return $SQL
}

##
# Détermine si une table est table fille d'un héritage
##
proc Katyusha_GenerationSQL_table_fille_ {id_table table} {
    global heritages
    
    set res 0
    foreach {k heritage} $heritages {
        set filles [dict get $heritage "filles"]
        foreach {k fille} $filles {
            if {$fille == $id_table} {
                set res 1
            }
        }
    }
    
    return $res
}

##
# Détermine si une table est table mère d'un héritage
##
proc Katyusha_GenerationSQL_table_mere_ {id_table table} {
    global heritages
    
    set res 0
    set id_heritage -1
    foreach {k heritage} $heritages {
        set mere [dict get $heritage "mere"]
        if {$mere == $id_table} {
            set res 1
            set id_heritage $k
        }
    }
    
    return [list $id_heritage $res]
}

##
#
##
proc Katyusha_GenerationSQL_table_mere_ajout_attributs_filles {attributs id_heritage nom_table_mere} {
    global heritages
    global tables
    
    set heritage [dict get $heritages $id_heritage]
    set filles [dict get $heritage "filles"]
    set id_attribut [expr [Katyusha_Attributs_dernier_id $attributs] + 1]
    # Balayage des tables filles
    foreach {k id_fille} $filles {
        set fille [dict get $tables $id_fille]
        set attributs_fille [dict get $fille "attributs"]
        foreach {kk attribut} $attributs_fille {
            dict set attributs $id_attribut [dict create "nom" [dict get $attribut "nom"] "type" [dict get $attribut "type"] "complement_type" [dict get $attribut "complement_type"] "taille" 0 "pk" 0 "null" 1 "auto" 0 "valeur" "null"]
            set id_attribut [expr $id_attribut + 1] 
        }
    }
    
    # Ajout de l'attribut discriminant
    dict set attributs $id_attribut [dict create "nom" "type_$nom_table_mere" "type" "varchar" "complement_type" "" "taille" 0 "pk" 0 "null" 0 "auto" 0 "valeur" $nom_table_mere]
    
    return $attributs
}

##
# Construit le script SQL des tables
##
proc Katyusha_tables_sql {tables sgbd} {
    set SQL ""
    # Balayage des tables
    foreach {k table} $tables {
        # On ne génère le script sql de la table que si elle n'est pas table fille d'un héritage
        if {[Katyusha_GenerationSQL_table_fille_ $k $table] == 0} {
        set nom_table [dict get $table "nom"]
        puts "Génération de la table : $nom_table"
        set attributs_table [dict get $table "attributs"]
        # Si la table en question est table mère d'un héritage, elle aspire les attributs de ses filles
        # Et on lui ajoute un attribut discriminant
        set table_mere [Katyusha_GenerationSQL_table_mere_ $k $table]
        if {[lindex $table_mere 1] == 1} {
            set attributs_table [Katyusha_GenerationSQL_table_mere_ajout_attributs_filles $attributs_table [lindex $table_mere 0] $nom_table]
        }
        # Si pas de description, ""
        if {[dict exists $table "description"]} {
            set description_table [dict get $table "description"]
        } else {
            set description_table ""
        }
        set attributs_sql ""
        set pks_sql ""
        set pks [list]
        # Balayage des attributs de la table
        foreach {k attribut} $attributs_table {
            if {[dict get $attribut "pk"] == 1} {
                lappend pks [dict get $attribut nom]
                set null_attribut 0
            }
            set attribut_sql "    [Katyusha_GenerationSQL_attribut $k $attribut $sgbd]"
            if {$attributs_sql != ""} {
                set attributs_sql "$attributs_sql, \n$attribut_sql"
            } else {
                set attributs_sql "$attributs_sql$attribut_sql"
            }
        }
        # Balayage des clefs primaires de la table
        foreach pk $pks {
            if {$pks_sql == ""} {
                set pks_sql ", \n    PRIMARY KEY($pk"
            } else {
                set pks_sql "$pks_sql, $pk"
            }
        }
        if {$pks_sql != ""} {
            set pks_sql "$pks_sql)"
        }
        # Construit le script de la table
        if {$description_table != "" && $description_table != "\n"} {
            set description_table "\n/*\nTODO\n*/\n"
        }
        set table "/* Table : $nom_table */\n$description_table\nCREATE TABLE $nom_table (\n$attributs_sql$pks_sql\n) ;\n\n\n"
        # Ajoute le script de la table au script général
        set SQL "$SQL$table"
        }
    }
    return $SQL
}

##
# Renvoie une liste contenant la ou les clefs primaire(s) d'une table selon son nom
##
proc Katyusha_pk_table {nom_table sgbd} {
    global tables
    set attributs [list]
    foreach {k table} $tables {
        set nom_table_liste [dict get $table "nom"]
        # Si le nom correspond
        if {$nom_table_liste == $nom_table} {
            set attributs_table [dict get $table "attributs"]
            # Balayage des attributs à la recherche des clefs primaires
            foreach {k attribut} $attributs_table {
                set pk_attribut [dict get $attribut "pk"]
                # Ajoute le nom de la table (s'il n'est pas déjà inclu) pour une meilleure lisibilité
                # et éviter les doublons
                # Par example :
                #       table.id devient table.id_table
                set nom_attribut [dict get $attribut "nom"]
                dict set attribut "nom_origine" $nom_attribut
                # Supprime les incrémentations automatiques
                dict set attribut "auto" 0
                dict set attribut "valeur" ""
                if {[string first $nom_table $nom_attribut] == -1} {
                    dict set attribut "nom" "$nom_attribut\_$nom_table"
                }
                if {$pk_attribut == 1} {
                    dict set attributs [dict size $attributs] $attribut
                }
            }
        }
    }
    return $attributs
}

##
#
##
proc Katyusha_ajout_fk_table {table_lien table_lier pk_table_lier sgbd} {
    set SQL ""
    foreach {k pk} $pk_table_lier {
        set nom_attribut [dict get $pk "nom"]
        set nom_origine_attribut [dict get $pk "nom_origine"]
        set type_attribut [dict get $pk "type"]
        set taille_attribut [dict get $pk "taille"]
        set auto_attribut [dict get $pk "auto"]
        # Si l'attribut de la table d'origine est serial, le type ici est forcément un entier
        if {$auto_attribut == 1 && $sgbd == "postgres"} {
            set type_attribut "integer"
        }
        set SQL "$SQL\nALTER TABLE $table_lien ADD COLUMN $nom_attribut [string toupper $type_attribut] NOT NULL ;"
        set SQL "$SQL\nALTER TABLE $table_lien ADD CONSTRAINT FK_$table_lier\_$nom_attribut FOREIGN KEY \($nom_attribut\) REFERENCES $table_lier\($nom_origine_attribut\) ;\n"
    }
    return $SQL
}

##
# Détermine si une nouvelle table doit être créer
#       1 pour une nouvelle table
#       0 sinon
##
proc Katyusha_GenerationSQL_liens_n_table {liens} {
    set n_table 1
    
    if {[dict size $liens] == 2} {
        foreach {k lien} $liens {
            set table_lien [lindex $lien 0]
            set n_lien [lindex $lien 1]
            if {$n_table == 1} {
                if {$n_lien == "1.1"} {
                    set n_table 0
                } else {
                }
            }
        }
    }
    
    return $n_table
}

##
# Si une nouvelle table doit être créé, récupère les clefs primaires des tables liés pour en faire des colones de la table créée
##
proc Katyusha_GenerationSQL_attributs_n_table {liens sgbd} {
    set SQL ""
    set attributs [dict create]
    foreach {k lien} $liens {
        set table_lien [lindex $lien 0]
        set n_lien [lindex $lien 1]
        if {$n_lien == "0.n" || $n_lien == "0.1" || $n_lien == "1.n" || $n_lien == "n.n"} {
            # Liste des clefs primaires de la table
            set liste_pk_table [Katyusha_pk_table $table_lien $sgbd]
            # Balayage des clefs primaires
            foreach {kk attribut} $liste_pk_table {
                dict set attributs [dict size $attributs] $attribut
            }
        }
    }
    return $attributs
}

##
# Retourne le code SQL pour une relation
##
proc Katyusha_GenerationSQL_relation {relation sgbd} {
    set SQL ""
    set nom_relation [dict get $relation "nom"]
    # Au cas ou il faudrait créer une table spécifique à cette relation
    set attributs [dict create]
    set c 0
    # Balayage des liens
    set liens [dict get $relation "liens"]
    
    # Détermine si une nouvelle table doit être créée
    set n_table [Katyusha_GenerationSQL_liens_n_table $liens]
    
    if {$n_table == 1} {
        # Créé les attributs de la nouvelle table
        dict set table_tmp "nom" $nom_relation
        dict set table_tmp "description" ""
        dict set table_tmp "attributs" [Katyusha_GenerationSQL_attributs_n_table $liens $sgbd]
    } else {
        set table_tmp "null"
    }
    return [list $SQL $table_tmp]
}

proc Katyusha_GenerationSQL_relations {relations sgbd} {
    set SQL ""
    set tables_tmp [list]
    
    foreach {k relation} $relations {
        set sql_table [lindex [Katyusha_GenerationSQL_relation $relation $sgbd] 0]
        set table [lindex [Katyusha_GenerationSQL_relation $relation $sgbd] 1]
        if {$table != "null"} {
            lappend tables_tmp $table
        } else {
            set liens [dict get $relation "liens"]
            set lien1 [dict get $liens 0]
            set lien2 [dict get $liens 1]
            set table_lien [lindex $lien1 0]
            set table_liee [lindex $lien2 0]
            set pk_table_liee [Katyusha_pk_table $table_liee $sgbd]
            Katyusha_ajout_fk_table $table_lien $table_liee $pk_table_liee $sgbd
        }
        set SQL "$SQL$sql_table"
    }
    
    return [list $SQL $tables_tmp]
}

##
# Ajoute un drop et create au script SQL
##
proc Katyusha_GenerationSQL_base {sgbd} {
    global CONFIGS
    global MCD
    
    set SQL ""
    
    # Si drop
    if {$MCD(drop) == 1} {
        set SQL "$SQL[Katyusha_SQL_drop_base $MCD(nom) $sgbd]"
    }
    
    return $SQL
}

proc Katyusha_GenerationSQL_liens_egaux {liens} {
    set res 0
    set lien1 "null"
    
    foreach {k lien} $liens {
        set lien2 [lindex $lien 1]
        if {$lien1 == $lien2} {
            set res 1
        } else {
            set res 0
        }
        set lien1 $lien2
    }
    
    return $res
}

##
# Créé le code SQL des clefs étrangères à ajouter
##
proc Katyusha_GenerationSQL_FK {tables relations sgbd} {
    set SQL ""
    foreach {k relation} $relations {
        set nom_relation [dict get $relation "nom"]
        # Récupère et analyse les liens de la relation
        set liens [dict get $relation "liens"]
        set n_table [Katyusha_GenerationSQL_liens_n_table $liens]
        
        # Si une nouvelle table doit être créée
        if {$n_table == 1} {
            foreach {k lien} $liens {
                set table_lien [lindex $lien 0]
                set n_lien [lindex $lien 1]
                set pk_table_lien [Katyusha_pk_table $table_lien $sgbd]
                foreach {k pk} $pk_table_lien {
                    set nom_pk [dict get $pk "nom"]
                    set nom_origine_pk [dict get $pk "nom_origine"]
                    set SQL "$SQL[Katyusha_SQL_ajout_fk $nom_relation $table_lien $nom_pk $nom_origine_pk]"
                }
            }
        } else {
            set liens [dict get $relation "liens"]
            set lien1 [dict get $liens 0]
            set lien2 [dict get $liens 1]
            if {[Katyusha_GenerationSQL_liens_egaux $liens] == 1} {
                set table_lien [lindex $lien1 0]
                set table_liee [lindex $lien2 0]
            } else {
                set type_lien1 [lindex $lien1 1]
                if {$type_lien1 == "1.1"} {
                    set table_lien [lindex $lien2 0]
                    set table_liee [lindex $lien1 0]
                } else {
                    set table_lien [lindex $lien1 0]
                    set table_liee [lindex $lien2 0]
                }
            }
            set pk_table_liee [Katyusha_pk_table $table_liee $sgbd]
            foreach {k pk} $pk_table_liee {
                set nom_pk [dict get $pk "nom"]
                set nom_origine_pk [dict get $pk "nom_origine"]
                set type_pk [dict get $pk "type"]
                if {[lsearch [Katyusha_SQL_liste_types_taille] [string tolower $type_pk]] != -1} {
                    set taille_pk [dict get $pk "taille"]
                } else {
                    set taille_pk "null"
                }
                set SQL "$SQL[Katyusha_SQL_ajout_colone $table_lien $nom_pk $type_pk $taille_pk]"
                set SQL "$SQL[Katyusha_SQL_ajout_fk $table_lien $table_liee $nom_pk $nom_origine_pk]"
            }
        }
    }
    return $SQL
}

##
# Génère le script SQL du MCD
##
proc Katyusha_generation_sql {sgbd} {
    global tables
    set tables_tmp $tables
    global relations
    set relations_tmp $relations
    global id_projet
    global rep_mcd
    global version
    global fk
    global MCD
    
    puts "Génération du script SQL pour $sgbd"
    
    set SQL "/*\nScript généré automatiquement par Katyusha MCD v$version pour $sgbd\n*/\n\n"
    
    set sql_base [Katyusha_GenerationSQL_base $sgbd]
    set SQL "$SQL$sql_base"
    
    set retour_relations [Katyusha_GenerationSQL_relations $relations_tmp $sgbd]
    set relations_sql [lindex $retour_relations 0]
    # Ajoute les nouvelles tables créées pour les relations
    foreach table_tmp [lindex $retour_relations 1] {
        set c [expr [Katyusha_Tables_dernier_id $tables_tmp] + 1]
        dict set tables_tmp $c $table_tmp
    }
    set tables_sql [Katyusha_tables_sql $tables_tmp $sgbd]
    set SQL "$SQL$tables_sql$relations_sql"
    set SQL "$SQL[Katyusha_GenerationSQL_FK $tables $relations $sgbd]"
    # Défini le chemin d'enregistrement du fichier SQL
    if {$id_projet == -1} {
        set id_projet [llength [glob -nocomplain -dir $rep_mcd "projet*"]]
        file mkdir "$rep_mcd/projet$id_projet"
    }
    set id_fichier [llength [glob -nocomplain -dir "$rep_mcd/projet$id_projet" "$MCD(nom)*.sql"]]
    set fichier "$rep_mcd/projet$id_projet/$MCD(nom)$id_fichier\-$sgbd.sql"
    # Enregistre dans le fichier
    set stream [open $fichier "w+"]
    puts $stream $SQL
    close $stream
    
    puts "Script SQL généré avec succès : $fichier"
    
    return [list $SQL $fichier]
}

##
# Vérifie le MCD avant génération du script SQL
##
proc Katyusha_verification_mcd_sql {sgbd} {
    set verif [Katyusha_verification_mcd]
    set res_sql [list 0 0]
    if {[lindex $verif 0] == 0} {
        # Informe qu'il y a un problème
        set errs [lindex $verif 1]
        set res_info [tk_messageBox -icon warning -type ok -message "Il y a une erreur dans le MCD qui empêche la génération du script SQL pour $sgbd"]
    } else {
        set errs "null"
        set res_sql [Katyusha_generation_sql $sgbd]
    }
    return [list [lindex $res_sql 0] [lindex $res_sql 1] $errs]
}
