#!/usr/bin/env tclsh
##
# Créé une fenêtre demandant le nom de la nouvelle table à créer
# Puis, envoie vers une fenêtre pour créer les attributs de la table
# TODO : Vérifier que le nom de la table ne contient pas de caractères interdits
##
proc INTERFACE_ajout_table_nom {x y} {
    global LOCALE
    global coords
    global IMG
    # Transfert des coordonnées
    set coords [list $x $y]
    set f ".fen_ajout_table"
    # Détruit la fenêtre si elle existe déjà
    if {[winfo exists $f]} {
        destroy $f
    }
    toplevel $f
    # Icone de la fenêtre
    wm iconphoto $f $IMG(logo)
    label $f.lnom -text $LOCALE(nom_de_la_nouvelle_table)
    pack $f.lnom
    entry $f.enom
    pack $f.enom
    frame $f.commandes
        button $f.commandes.ok -text $LOCALE(valider) -image $IMG(valider) -compound left -command {
            set nom_table [.fen_ajout_table.enom get]
            # Vérifie que le nom entré n'est pas vide
            if {$nom_table == ""} {
                # Si le nom de la table est vide, on informe l'utilisateur
                set g [tk_messageBox -message $LOCALE(le_nom_de_la_table_ne_peut_pas_etre_vide) -type ok]
                puts $LOCALE(le_nom_de_la_table_ne_peut_pas_etre_vide)
            } else {
                # Sinon, on détruit celle là
                destroy .fen_ajout_table
                # Et on envoie la fenêtre suivante
                INTERFACE_ajout_table_attributs $nom_table
            }
        }
        button $f.commandes.ko -text $LOCALE(retour) -image $IMG(retour) -compound left -command {destroy .fen_ajout_table}
        pack $f.commandes.ok $f.commandes.ko -fill x -side left -padx 20
    pack $f.commandes -fill x
    # Titre le la présente fenêtre
    wm title $f $LOCALE(ajouter_une_table)
    # Mise à jour forcée de l'affichage graphique
    update
}

##
# Créé une fenêtre permettant de paramètrer la table : ajout de colones, clefs primaires, ...
# TODO : meilleur gestion de configuration des attributs
##
proc INTERFACE_ajout_table_attributs {nom_table} {
    global LOCALE
    global nombre_attributs
    global table_tmp
    global IMG
    # Initialise la table temporaire
    dict set table_tmp "nom" $nom_table
    dict set table_tmp "attributs" [dict create]
    # Réinitialise le nombre d'attributs
    set nombre_attributs 1
    set f ".fen_ajout_attributs_table"
    # Détruit la fenêtre si elle existe déjà
    if {[winfo exists $f]} {
        destroy $f
    }
    # Nouvelle fenêtre
    toplevel $f
    # Icone de la fenêtre
    wm iconphoto $f $IMG(logo)
    label $f.nom_table -text "$nom_table"
    pack $f.nom_table
    # Frame des attributs de la table
    frame $f.attribut_tmp
        frame $f.attribut_tmp.commandes -width 200
            button $f.attribut_tmp.commandes.ajout -text $LOCALE(ajouter_une_ligne) -image $IMG(ajouter) -command INTERFACE_ajout_table_attributs_ajout_ligne
            pack $f.attribut_tmp.commandes.ajout -side left
        pack $f.attribut_tmp.commandes
        frame $f.attribut_tmp.etiquettes
            entry $f.attribut_tmp.etiquettes.nom -textvariable LOCALE(nom) -state readonly -justify center
            entry $f.attribut_tmp.etiquettes.type -textvariable LOCALE(type) -state readonly -justify center
            entry $f.attribut_tmp.etiquettes.pk -textvariable LOCALE(pk) -state readonly -justify center
            entry $f.attribut_tmp.etiquettes.auto -textvariable LOCALE(auto) -state readonly -justify center
            entry $f.attribut_tmp.etiquettes.valeur -textvariable LOCALE(valeur) -state readonly -justify center
            pack $f.attribut_tmp.etiquettes.nom $f.attribut_tmp.etiquettes.type $f.attribut_tmp.etiquettes.pk $f.attribut_tmp.etiquettes.auto $f.attribut_tmp.etiquettes.valeur -side left
        pack $f.attribut_tmp.etiquettes
        frame $f.attribut_tmp.0
            entry $f.attribut_tmp.0.nom
            entry $f.attribut_tmp.0.type
            entry $f.attribut_tmp.0.pk
            entry $f.attribut_tmp.0.auto
            entry $f.attribut_tmp.0.valeur
            pack $f.attribut_tmp.0.nom $f.attribut_tmp.0.type $f.attribut_tmp.0.pk $f.attribut_tmp.0.auto $f.attribut_tmp.0.valeur -side left
        pack $f.attribut_tmp.0
    pack $f.attribut_tmp
    frame $f.description
        label $f.description.titre -text $LOCALE(description_de_la_table)
        frame $f.description.text_block
            text $f.description.text_block.t -height 5 -yscrollcommand {.fen_ajout_attributs_table.description.text_block.ysbar set}
            scrollbar $f.description.text_block.ysbar -orient vertical -command {.fen_ajout_attributs_table.description.text_block.t yview}
            pack $f.description.text_block.t -side left -fill x
            pack $f.description.text_block.ysbar -side left -fill y
        pack $f.description.titre $f.description.text_block -fill x
    pack $f.description -fill x
    # Frame des commandes générales
    frame $f.commandes
        button $f.commandes.ok -text $LOCALE(valider) -command {
            global nombre_attributs
            global coords
            set f ".fen_ajout_attributs_table"
            set c 0
            # Scan toutes les lignes d'ajout des attributs et conserve celle qui ne sont pas vides
            while {$c < $nombre_attributs} {
                set vide 1
                foreach a [list "nom" "type" "pk" "auto" "valeur"] {
                    if {[$f.attribut_tmp.$c.$a get] != ""} {
                        set vide 0
                    }
                    dict set tmp $a [$f.attribut_tmp.$c.$a get]
                }
                if {$vide == 0} {
                    lappend attributs $tmp
                }
                set c [expr $c+1]
            }
            # Coordonnées de la table
            dict set table_tmp "coords" $coords
            dict set table_tmp "attributs" $attributs
            dict set table_tmp "description" [$f.description.text_block.t get 1.0 end]
            destroy $f
            ajout_table $table_tmp
            unset table_tmp attributs tmp c a vide f
        }
        button $f.commandes.ko -text $LOCALE(retour) -command {destroy .fen_ajout_attributs_table}
        pack  $f.commandes.ok $f.commandes.ko -side left
    pack $f.commandes
    puts "Nouvelle table : $nom_table"
    # Titre le la présente fenêtre
    wm title $f $LOCALE(ajouter_des_attributs)
    # Mise à jour forcée de l'affichage graphique
    update
}

proc INTERFACE_ajout_table_attributs_ajout_ligne {} {
    global LOCALE
    global nombre_attributs
    global table_tmp
    set f ".fen_ajout_attributs_table"
    frame $f.attribut_tmp.$nombre_attributs
        entry $f.attribut_tmp.$nombre_attributs.nom
        entry $f.attribut_tmp.$nombre_attributs.type
        entry $f.attribut_tmp.$nombre_attributs.pk
        entry $f.attribut_tmp.$nombre_attributs.auto
        entry $f.attribut_tmp.$nombre_attributs.valeur
        pack $f.attribut_tmp.$nombre_attributs.nom $f.attribut_tmp.$nombre_attributs.type $f.attribut_tmp.$nombre_attributs.pk $f.attribut_tmp.$nombre_attributs.auto $f.attribut_tmp.$nombre_attributs.valeur -side left
    pack $f.attribut_tmp.$nombre_attributs
    set nombre_attributs [expr $nombre_attributs+1]
    update
}

##
# Affiche le nom et les attributs d'une table pour pouvoir la modifier
# TODO : tout
##
proc INTERFACE_modification_table {table} {
    global LOCALE
    global IMG_logo
    puts "Modification : $table"
}